@@ -115,6 +115,8 @@ Datatype and Object Properties
     :show-inheritance:
 .. autoclass:: cached_predicate
     :show-inheritance:
+.. autoclass:: cached_object_predicate
+    :show-inheritance:
 
 Core classes
 ------------
@@ -235,16 +237,37 @@ class object_predicate(predicate):
     >>> quebec.country = canada
     >>> for country in quebec.country: print [str(x) for x in country.label]
     ['Canada']
+
+    Try out some support for strings in place of class names for
+    self-referential classes.
+
+    >>> from ordf.namespace import FOAF
+    >>> class Person(Class):
+    ...     knows = object_predicate(FOAF.knows, "Person", globals())
+    ...
+    >>> bob = Person(EX.bob)
+    >>> alice = Person(EX.alice)
+    >>> bob.knows = alice
+    >>> [type(x) for x in bob.knows]
+    [<class 'ordf.vocab.owl.Person'>]
     """
-    def __init__(self, term, implementation=None):
+    def __init__(self, term, implementation=None, globals=None):
         if implementation is None:
-            implementation = Individual
-        assert issubclass(implementation, Individual), "expected an Individual, got an %s" % implementation
+            self._cls = Individual
+        if isinstance(implementation, basestring):
+            self._cls = implementation
+            self._globals = globals
+        else:
+            assert issubclass(implementation, Individual), "expected an Individual, got an %s" % implementation
+            self._cls = implementation
         super(object_predicate, self).__init__(term)
-        self._cls = implementation
+
     def _get(self, obj):
+        if isinstance(self._cls, basestring):
+            self._cls = self._globals[self._cls]
         for identifier in super(object_predicate, self)._get(obj):
-            yield self._cls(identifier, graph=obj.graph)
+            yield self._cls(identifier, graph=obj.graph, factoryGraph=obj.factoryGraph, 
+                            skipClassMembership=True)
 
 class cached_predicate(predicate):
     """
@@ -311,29 +334,43 @@ class cached_object_predicate(cached_predicate):
     >>> bob.graph.remove((bob.identifier, None, None))
     >>> for pet in bob.pet: print [str(x) for x in pet.label]
     ['Max']
+    >>> # prove that it is the *same* object
+    >>> for pet in bob.pet:
+    ...     assert id(pet) == id(max)
+    ...
     """
-    def __init__(self, term, implementation=None):
+    def __init__(self, term, implementation=None, globals=None):
         if implementation is None:
-            implementation = Individual
-        assert issubclass(implementation, Individual), "expected an Individual, got an %s" % implementation
+            self._cls = Individual
+        elif isinstance(implementation, basestring):
+            self._cls = implementation
+            self._globals = globals
+        else:
+            assert issubclass(implementation, Individual), "expected an Individual, got an %s" % implementation
+            self._cls = implementation
         super(cached_object_predicate, self).__init__(term)
-        self._cls = implementation
+
     def _set(self, obj, value):
         if not isinstance(value, list) and not isinstance(value, tuple):
             value = [value]
+        cached_values = self._get(obj)
         for x in value:
             if isinstance(x, URIRef) or isinstance(x, BNode):
                 x = self._cls(value, graph=obj.graph)
             assert isinstance(x, Individual), "expected an Individual, got an %s" % x
-            obj.graph.add((obj.identifier, self.term, x.identifier))
-        _cache = self._cache(obj)
-        if self.term in _cache: del _cache[self.term]
+            if x not in cached_values:
+                cached_values.append(x)
+                obj.graph.add((obj.identifier, self.term, x.identifier))
+
     def _get(self, obj):
+        if isinstance(self._cls, basestring):
+            self._cls = self._globals[self._cls]
         _cache = self._cache(obj)
         if self.term in _cache:
             return _cache[self.term]
         results = [
-            self._cls(x, graph=obj.graph)
+            self._cls(x, graph=obj.graph, factoryGraph=obj.factorGraph,
+                      skipClassMembership=True)
             for x in super(cached_object_predicate, self)._get(obj)
             ]
         _cache[self.term] = results
@@ -531,7 +568,7 @@ class Individual(object):
     .. automethod:: replace
     .. automethod:: serialize
     """
-    def __init__(self, identifier=None, graph=None, factoryGraph=None):
+    def __init__(self, identifier=None, graph=None, factoryGraph=None, **kw):
         self.__identifier = identifier is not None and identifier or BNode()
         if factoryGraph is not None:
             self.factoryGraph = factoryGraph
@@ -641,9 +678,6 @@ class AnnotatibleTerms(Individual):
 
         Similarly for *rdfs:seeAlso*
     """
-    def __init__(self, identifier, graph=None, factoryGraph=None):
-        super(AnnotatibleTerms, self).__init__(identifier, graph=graph, factoryGraph=factoryGraph)
-
     label = predicate(RDFS.label)
     comment = predicate(RDFS.comment)
     seeAlso = predicate(RDFS.seeAlso)
@@ -814,7 +848,7 @@ def CastClass(c,graph=None):
                 else:
                     return BooleanClass(classOrIdentifier(c),operator=p,graph=graph)
             #assert (classOrIdentifier(c),RDF.type,OWL.Class) in graph
-            return Class(classOrIdentifier(c),graph=graph,skipOWLClassMembership=True)
+            return Class(classOrIdentifier(c),graph=graph,skipClassMembership=True)
     
 class Class(AnnotatibleTerms):
     """
@@ -866,9 +900,10 @@ class Class(AnnotatibleTerms):
     def __init__(self, identifier=None, graph=None, factoryGraph=None, factoryClass=Individual,
                  subClassOf=None,equivalentClass=None,
                  disjointWith=None,complementOf=None,
-                 skipOWLClassMembership=False,comment=None):
+                 skipClassMembership=False,
+                 comment=None):
         super(Class, self).__init__(identifier, graph=graph, factoryGraph=factoryGraph)
-        if not skipOWLClassMembership and (self.identifier,RDF.type,OWL.Class) not in self.graph and \
+        if not skipClassMembership and (self.identifier,RDF.type,OWL.Class) not in self.graph and \
            (self.identifier,RDF.type,OWL.Restriction) not in self.graph:
             self.graph.add((self.identifier,RDF.type,OWL.Class))
         self.factoryClass    = factoryClass
@@ -1026,7 +1061,7 @@ class Class(AnnotatibleTerms):
             class _xClass(Individual):
                 def __new__(self, *av, **kw):
                     kwa=kw.copy()
-                    kwa["skipOWLClassMembership"] = True
+                    kwa["skipClassMembership"] = True
                     return Class(*av, **kwa)
             super(self.__class__, self).__init__(RDFS.subClassOf, _xClass)
     subClassOf = _subClassOf()
@@ -1079,7 +1114,7 @@ class Class(AnnotatibleTerms):
         
 #    def __iter__(self):
 #        for s in self.graph.subjects(predicate=RDFS.subClassOf,object=self.identifier):
-#            yield Class(s,skipOWLClassMembership=True)
+#            yield Class(s,skipClassMembership=True)
     
     def __repr__(self,full=False,normalization=True):
         """
@@ -1422,7 +1457,7 @@ class Restriction(Class):
                  identifier=None):
         super(Restriction, self).__init__(identifier,
                                           graph=graph,
-                                          skipOWLClassMembership=True)
+                                          skipClassMembership=True)
         if (self.identifier,
             OWL.onProperty,
             propertyOrIdentifier(onProperty)) not in graph:
